#R script file for the generation of metabolic heat maps from mass spectrometry data

#load data matrix into R
msdata <- read.table("massR.dat")

#extract mz-values as list
mzall<-msdata[,2]

#binning of mz-data with fixed intervals
mzbin1000 <- cut(mzall,breaks=seq(50, 2000, by = 1))

#convert resulting binning factor to vector
mzbin1000vector<-as.vector(mzbin1000)

#add binning information to data frame
msdata$bin<-mzbin1000vector

#remove data outside binning range
msdata <- msdata[!is.na(msdata[,4]),]
mzall<-msdata[,2]

#Generate non-redundant list of ms-files
msfiles<-unique(msdata[,1])
msfilesvector<-as.vector(msfiles)

#Non-redundant list of bins
msbins<-unique(msdata[,4])

#List of mz-bins to be extracted from the outputmatrix
mztargets<-c("(123,124]","(124,125]","(138,139]","(286,287]","(399,400]","(664,665]","(665,666]")
mztargetnames<-c("m/z 123-124 (nicotinamide)","m/z 124-125 (nicotinic acid)", "m/z 138-138 (trigonelline)", "m/z 286-287 (N-glucosylnicotinate)","m/z 399-400 (SAM)","m/z 664-665 (NAD)","m/z 665-666 (NHD)")

#sample targets: samples used for the target matrix/ target heatmap
#default: all samples
#sampletargets<-msfilesvector

#defined samples; here, only coffees and unprocessed green tea
sampletargets<-c("Ca-EC1_1","Ca-EC1_2","Ca-EC1_3","Ca-IN_1","Ca-IN_2","Ca-IN_3","Ca-MA_1","Ca-MA_2","Ca-MA_3","Cc-36_1","Cc-36_2","Cc-36_3","Cc-37_1","Cc-37_2","Cc-37_3","Cc-38_1","Cc-38_2","Cc-38_3","Cc-K_1","Cc-K_2","Cc-K_3","Cs-Green_1","Cs-Green_2","Cs-Green_3","Ec-Coca_1","Ec-Coca_2","Ec-Coca_3")

#generate empty output matrix
outputmatrix <- matrix(0, nrow = length(msfilesvector), ncol = length(msbins))
rownames(outputmatrix)<-msfilesvector
colnames(outputmatrix)<-msbins

#fill matrix with data 
i=j=k=l=m=1
for (i in 1:length(mzall)) {for (l in 1:length(msfilesvector)) {if (msdata[i,1] == msfilesvector[l]) {j=l}}; for (m in 1:length(msbins)) {if (msdata[i,4] == msbins[m]) {k=m}}; outputmatrix[j,k]<-msdata[i,3]}

#targetmatrix with selected compounds
targetmatrix<-outputmatrix[sampletargets,mztargets]
#default names:
#colnames(targetmatrix)<-mztargets

#custom compound names
colnames(targetmatrix)<-mztargetnames

#spectra normalization
install.packages("vegetarian")
library(vegetarian)
normoutputmatrix<-normalize.rows(outputmatrix)
rownames(normoutputmatrix)<-msfilesvector
colnames(normoutputmatrix)<-msbins

#find maximal value for each column (mz-bin)
columnmax<-apply(normoutputmatrix, 2, max)

#sort and select the 75 most intense values
colmax<-sort(columnmax,decreasing=TRUE)
colmaxselection<-as.matrix(colmax[1:75])
colmaxselection=as.vector(rownames(colmaxselection))

#limiting matrix to selection
normoutputmatrix<-normoutputmatrix[,colmaxselection]

#Generation of heat maps
install.packages("pheatmap")
library(pheatmap)

#transformation of matrices
tnormoutputmatrix<-t(normoutputmatrix)
ttargetmatrix<-t(targetmatrix)

#Un-targeted metabolic heat map
ann_text = data.frame(Plant = c("C. arabica","C. arabica","C. arabica","C. arabica","C. arabica","C. arabica","C. arabica","C. arabica","C. arabica","C. canephora","C. canephora","C. canephora","C. canephora","C. canephora","C. canephora","C. canephora","C. canephora","C. canephora","C. canephora","C. canephora","C. canephora","Black tea", "Black tea","Black tea","Red tea","Red tea","Red tea","Green tea","Green tea","Green tea","C. canephora","C. canephora","C. canephora","Coca tea","Coca tea","Coca tea"))
rownames(ann_text) = msfiles

Plant = c("yellow", "red", "black","lightgreen","green","darkgreen")
names(Plant) = c("Coca tea","Red tea","Black tea","Green tea","C. arabica","C. canephora")

ann_colors = list(Plant = Plant)

png('metabolic-heatmap-75.png', width = 3200, height = 3200, units = "px", pointsize = 6, bg = "white", res = 600)
pheatmap(tnormoutputmatrix,scale="row",cluster_cols = TRUE, cluster_rows = TRUE, clustering_distance_rows="euclidean", clustering_distance_cols="euclidean", clustering_methode="complete",cellwidth = NA, cellheight = NA, fontsize=5, annotation = ann_text, annotation_colors = ann_colors,filename="metabolic-heatmap-75.pdf")
dev.off()

#Targeted metabolic heat map
targetann_text = data.frame(samplePlant = c("C. arabica","C. arabica","C. arabica","C. arabica","C. arabica","C. arabica","C. arabica","C. arabica","C. arabica","C. canephora","C. canephora","C. canephora","C. canephora","C. canephora","C. canephora","C. canephora","C. canephora","C. canephora","C. canephora","C. canephora","C. canephora","Green tea","Green tea","Green tea","Coca tea","Coca tea","Coca tea"))
rownames(targetann_text) = sampletargets

samplePlant = c("yellow","lightgreen","green","darkgreen")
names(samplePlant) = c("Coca tea","Green tea","C. arabica","C. canephora")

targetann_colors = list(samplePlant = samplePlant)

png('target-heatmap.png', width = 3200, height = 3200, units = "px", pointsize = 6, bg = "white", res = 600)
pheatmap(ttargetmatrix,scale="row",cluster_cols = FALSE, cluster_rows = TRUE, clustering_distance_rows="euclidean", clustering_distance_cols="binary", clustering_methode="complete",cellwidth = NA, cellheight = NA,fontsize=5,annotation =targetann_text, annotation_colors = targetann_colors,legend=FALSE,filename="target-heatmap.pdf")
dev.off()

#correlation between two metabolites
correlationdata<-as.data.frame(outputmatrix)
correlationcompounds<-c("(138,139]","(665,666]")
correlationsamples<-c("Ca-EC1_1","Ca-EC1_2", "Ca-EC1_3", "Ca-IN_1", "Ca-IN_2", "Ca-IN_3", "Ca-MA_1","Ca-MA_2", "Ca-MA_3")
correlationdata<-correlationdata[correlationsamples,correlationcompounds]

names(correlationdata)[names(correlationdata)=="(138,139]"] <- "trigonelline"
names(correlationdata)[names(correlationdata)=="(665,666]"] <- "NHD"
attach(correlationdata)

png("correlationplot.png", width = 3200, height = 3200, units = "px", pointsize = 12, bg = "white", res = 600)
plot(NHD,trigonelline,type="p",main="Correlation plot", xlab="m/z 665-666 (NHD)",ylab="m/z 138-139   (trigonelline)",pch=21,col="black",bg="blue")
abline(lm(trigonelline~NHD))
#draw error lines
lm.trig<-lm(trigonelline~NHD)
segments(NHD,fitted(lm.trig),NHD,trigonelline)
dev.off()

detach(correlationdata)


#preparing data for boxplots
organisms<-strsplit(rownames(outputmatrix),"_")
orgname<-as.vector(unlist(organisms))
n <- length(orgname)
orgname1<-orgname[seq(1, n, 2)]
orgname2<-orgname[seq(2, n, 2)] 

boxplotdata<-as.data.frame(outputmatrix)
boxplotdata$organismname<-orgname1

names(boxplotdata)[names(boxplotdata)=="(134,135]"] <- "c134"
names(boxplotdata)[names(boxplotdata)=="(286,287]"] <- "c286"
names(boxplotdata)[names(boxplotdata)=="(304,305]"] <- "c304"
names(boxplotdata)[names(boxplotdata)=="(872,873]"] <- "c872"

attach(boxplotdata)

#array of boxplots

#set output file
#pdf("boxplotarray.pdf")
png("boxplotarray.png", width = 3200, height = 3200, units = "px", pointsize = 6, bg = "white", res = 600)

par(mfrow=c(2,2),mar = c(2.5,5,2,1))

boxplot(c134 ~ organismname, horizontal=TRUE,las=1, main = "m/z 134-135 (aspartate)", xlab = "intensity", col="blue")
boxplot(c286 ~ organismname, horizontal=TRUE,las=1, main = "m/z 286-287 (N-glucosylnicotinate)", xlab = "intensity", col="red")
boxplot(c304 ~ organismname, horizontal=TRUE,las=1, main = "m/z 304-305 (cocaine)", xlab = "intensity", col="yellow")
boxplot(c872 ~ organismname, horizontal=TRUE,las=1, main = "m/z 872-873 (chlorophyll a')", xlab = "intensity", col="green")

dev.off()

detach(boxplotdata)

#Generation of correlation plots
#re-using variables! The correlation matrix must even or smaller than the heatmap matrix
png("correlationmatrix-20.png", width = 3200, height = 3200, units = "px", pointsize = 6, bg = "white", res = 600)
colmaxselection<-as.matrix(colmax[1:20])
colmaxselection=as.vector(rownames(colmaxselection))
normoutputmatrix<-normoutputmatrix[,colmaxselection]
correlationmatrix=cor(normoutputmatrix)
pairs(correlationmatrix)
dev.off()
